import 'package:flutter/cupertino.dart';

class CaptainRouteParser {
  CaptainRouteParser({
    this.routeMap = const <String, Page Function(CaptainRouteData routeData)>{},
  }) {
    _buildRouteTree();
  }

  void _buildRouteTree() {}

  final Map<String, Page Function(CaptainRouteData routeData)> routeMap;

  Page parseRouteMapToPage(String routeInformationPath) {
    final List<CaptainRouteData> _routeDatas = [];
    for (final _uri in routeMap.keys) {
      _routeDatas.add(CaptainRouteData(uri: _uri));
    }
    int _segmentIndex = 0;
    Uri _routeInformationUri = Uri(path: routeInformationPath);
    while (_routeDatas.length > 1) {
      for (final _routeData in _routeDatas) {
        final bool _match = _routeData.matchesRouteInformationSegment(
            _routeInformationUri.pathSegments[_segmentIndex], _segmentIndex);
        if (_match == false) {
          _routeDatas.remove(_routeData);
        }
      }
      _segmentIndex++;
    }
    return routeMap[_routeDatas.first.uri]!(_routeDatas.first);
  }
}

class CaptainRouteData {
  CaptainRouteData({
    required this.uri,
  }) {
    _parsedUri = Uri(path: uri);
  }
  final String uri;

  late final Uri _parsedUri;

  T? queryParameter<T>(String id, [T Function(String queryData)? parser]) {
    final String? _queryData = _parsedUri.queryParameters[id];
    if (_queryData == null) {
      return null;
    }
    if (parser != null) {
      return parser(_queryData);
    }
    return _parsedUri.queryParameters[id] as T;
  }
}

class CaptainContext {}

extension _SegmentMatcher on CaptainRouteData {
  bool matchesRouteInformationSegment(String matchSegment, int segmentIndex) {
    final String _segment = _parsedUri.pathSegments[segmentIndex];
    if (_segment.startsWith('*') || _segment.startsWith(':')) {
      return true;
    }
    return _segment == matchSegment;
  }
}

// routenotfoundhandler
// routetree
